diff --git a/modules/access/qtcapture.m b/modules/access/qtcapture.m
index f6665eb..f461968 100644
--- a/modules/access/qtcapture.m
+++ b/modules/access/qtcapture.m
@@ -36,10 +36,14 @@
 #include <vlc_demux.h>
 #include <vlc_interface.h>
 #include <vlc_dialog.h>
+#include <vlc_access.h>
 
 #import <QTKit/QTKit.h>
 #import <CoreAudio/CoreAudio.h>
 
+#define QTC_WIDTH 640
+#define QTC_HEIGHT 480
+
 /*****************************************************************************
 * Local prototypes
 *****************************************************************************/
@@ -123,6 +127,7 @@ vlc_module_end ()
 {
     CVImageBufferRef imageBuffer;
     mtime_t pts;
+	void * pixels;
 
     if(!currentImageBuffer || currentPts == previousPts )
         return 0;
@@ -130,17 +135,22 @@ vlc_module_end ()
     @synchronized (self)
     {
         imageBuffer = CVBufferRetain(currentImageBuffer);
-        pts = previousPts = currentPts;
-
-        CVPixelBufferLockBaseAddress(imageBuffer, 0);
-        void * pixels = CVPixelBufferGetBaseAddress(imageBuffer);
-        memcpy( buffer, pixels, CVPixelBufferGetBytesPerRow(imageBuffer) * CVPixelBufferGetHeight(imageBuffer) );
-        CVPixelBufferUnlockBaseAddress(imageBuffer, 0);
+        if(imageBuffer){
+            pts = previousPts = currentPts;
+            CVPixelBufferLockBaseAddress(imageBuffer, 0);
+            pixels = CVPixelBufferGetBaseAddress(imageBuffer);
+            if(pixels)
+                memcpy( buffer, pixels, CVPixelBufferGetBytesPerRow(imageBuffer) * CVPixelBufferGetHeight(imageBuffer));
+            CVPixelBufferUnlockBaseAddress(imageBuffer, 0);
+        }
+            
     }
-
     CVBufferRelease(imageBuffer);
 
-    return currentPts;
+	if(pixels)
+		return currentPts;
+	else
+		return 0;
 }
 
 @end
@@ -196,10 +206,17 @@ static int Open( vlc_object_t *p_this )
     int i_width;
     int i_height;
     int result = 0;
+    char *psz_uid = NULL;
 
     /* Only when selected */
     if( *p_demux->psz_access == '\0' )
         return VLC_EGENERIC;
+
+    if(p_demux->psz_path && *p_demux->psz_path ){
+        psz_uid = strdup(p_demux->psz_path);
+    }
+    msg_Dbg( p_demux, "qtcapture uid = %s", psz_uid );
+    NSString *qtk_currdevice_uid = [[NSString alloc] initWithFormat:@"%s", psz_uid];
     
     NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
 
@@ -213,13 +230,40 @@ static int Open( vlc_object_t *p_this )
     p_demux->p_sys = p_sys = calloc( 1, sizeof( demux_sys_t ) );
     if( !p_sys )
         return VLC_ENOMEM;
-    
+    msg_Dbg( p_demux, "qtcapture : uid = %s", [qtk_currdevice_uid UTF8String]);
+    NSArray *myVideoDevices = [[[QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeVideo] arrayByAddingObjectsFromArray:[QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeMuxed]] retain];
+    if([myVideoDevices count] == 0)
+    {
+        dialog_FatalWait( p_demux, _("No Input device found"),
+                         _("Your Mac does not seem to be equipped with a suitable input device. "
+                           "Please check your connectors and drivers.") );
+        msg_Err( p_demux, "Can't find any Video device" );
+        
+        goto error;
+    }
+    int ivideo;
+    for(ivideo = 0; ivideo < [myVideoDevices count]; ivideo++){
+        QTCaptureDevice *qtk_device;
+        qtk_device = [myVideoDevices objectAtIndex:ivideo];
+        msg_Dbg( p_demux, "qtcapture %d/%d %s %s", ivideo, [myVideoDevices count], [[qtk_device localizedDisplayName] UTF8String], [[qtk_device uniqueID] UTF8String]);
+        if([[[qtk_device uniqueID]stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] isEqualToString:qtk_currdevice_uid]){
+            break;
+        }
+    }    
+        
     memset( &fmt, 0, sizeof( es_format_t ) );    
     
     QTCaptureDeviceInput * input = nil;
     NSError *o_returnedError;
-
-    p_sys->device = [QTCaptureDevice defaultInputDeviceWithMediaType: QTMediaTypeVideo];
+    if(ivideo < [myVideoDevices count]){
+        p_sys->device = [myVideoDevices objectAtIndex:ivideo];
+    }
+    else
+    {
+        /* cannot found designated device, fall back to open default device */
+        msg_Dbg(p_demux, "Cannot found designated uid device as %s. Fall back to open default.", [qtk_currdevice_uid UTF8String]);
+        p_sys->device = [QTCaptureDevice defaultInputDeviceWithMediaType: QTMediaTypeVideo];
+    }
     if( !p_sys->device )
     {
         dialog_FatalWait( p_demux, _("No Input device found"),
@@ -258,21 +302,15 @@ static int Open( vlc_object_t *p_this )
     {
         camera_format = [format_array objectAtIndex: k];
 
-        NSLog( @"%@", [camera_format localizedFormatSummary] );
-        NSLog( @"%@",[[camera_format formatDescriptionAttributes] description] );
+        msg_Dbg( p_demux, "%s", [[camera_format localizedFormatSummary] UTF8String]);
+        msg_Dbg( p_demux, "%s",[[[camera_format formatDescriptionAttributes] description] UTF8String]);
     }
     if( [format_array count] )
         camera_format = [format_array objectAtIndex: 0];
     else goto error;
 
     int qtchroma = [camera_format formatType];
-    int chroma = qtchroma_to_fourcc( qtchroma );
-    if( !chroma )
-    {
-        msg_Err( p_demux, "Unknown qt chroma %4.4s provided by camera", (char*)&qtchroma );
-        goto error;
-    }
-
+    int chroma = VLC_CODEC_UYVY;
     /* Now we can init */
     es_format_Init( &fmt, VIDEO_ES, chroma );
 
@@ -280,6 +318,13 @@ static int Open( vlc_object_t *p_this )
     NSSize display_size = [[camera_format attributeForKey:QTFormatDescriptionVideoCleanApertureDisplaySizeAttribute] sizeValue];
     NSSize par_size = [[camera_format attributeForKey:QTFormatDescriptionVideoProductionApertureDisplaySizeAttribute] sizeValue];
 
+    encoded_size.width = QTC_WIDTH;
+    encoded_size.height = QTC_HEIGHT;
+    display_size.width = QTC_WIDTH;
+    display_size.height = QTC_HEIGHT;
+    par_size.width = QTC_WIDTH;
+    par_size.height = QTC_HEIGHT;
+    
     fmt.video.i_width = p_sys->width = encoded_size.width;
     fmt.video.i_height = p_sys->height = encoded_size.height;
     if( par_size.width != encoded_size.width )
@@ -288,11 +333,12 @@ static int Open( vlc_object_t *p_this )
         fmt.video.i_sar_den = encoded_size.width;
     }
 
-    NSLog( @"encoded_size %d %d", (int)encoded_size.width, (int)encoded_size.height );
-    NSLog( @"display_size %d %d", (int)display_size.width, (int)display_size.height );
-    NSLog( @"PAR size %d %d", (int)par_size.width, (int)par_size.height );
+    msg_Dbg(p_demux, "encoded_size %d %d", (int)encoded_size.width, (int)encoded_size.height );
+    msg_Dbg(p_demux, "display_size %d %d", (int)display_size.width, (int)display_size.height );
+    msg_Dbg(p_demux, "PAR size %d %d", (int)par_size.width, (int)par_size.height );
     
     [p_sys->output setPixelBufferAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
+        [NSNumber numberWithUnsignedInt:kCVPixelFormatType_422YpCbCr8], (id)kCVPixelBufferPixelFormatTypeKey,
         [NSNumber numberWithInt: p_sys->height], kCVPixelBufferHeightKey,
         [NSNumber numberWithInt: p_sys->width], kCVPixelBufferWidthKey,
         [NSNumber numberWithBool:YES], (id)kCVPixelBufferOpenGLCompatibilityKey,
diff --git a/modules/gui/macosx/open.h b/modules/gui/macosx/open.h
index c7f4ddc..93b40bf 100644
--- a/modules/gui/macosx/open.h
+++ b/modules/gui/macosx/open.h
@@ -159,6 +159,16 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
     IBOutlet id o_screen_height_stp;
     IBOutlet id o_screen_follow_mouse_ckb;
 
+    /* QTK support */
+    IBOutlet id o_qtk_view;
+    IBOutlet id o_qtk_lbl;
+    IBOutlet id o_qtk_long_lbl;
+    IBOutlet id o_qtk_device_pop;
+    IBOutlet id o_qtk_label_view;
+
+    NSArray         *qtkvideoDevices;
+    NSString        *qtk_currdevice_uid;
+    
     BOOL b_autoplay;
     id o_currentCaptureView;
     NSString *o_file_slave_path;
@@ -167,6 +177,9 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
 
 + (VLCOpen *)sharedInstance;
 
+- (NSArray *)qtkvideoDevices;
+- (void)qtkrefreshDevices;
+
 - (void)setSubPanel;
 - (void)openTarget:(int)i_type;
 - (void)tabView:(NSTabView *)o_tv didSelectTabViewItem:(NSTabViewItem *)o_tvi;
@@ -196,6 +209,7 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
 - (void)openCapture;
 - (void)showCaptureView: theView;
 - (IBAction)openCaptureModeChanged:(id)sender;
+- (IBAction)qtkChanged:(id)sender;
 - (IBAction)eyetvSwitchChannel:(id)sender;
 - (IBAction)eyetvLaunch:(id)sender;
 - (IBAction)eyetvGetPlugin:(id)sender;
diff --git a/modules/gui/macosx/open.m b/modules/gui/macosx/open.m
index d6dd66c..8455ebb 100644
--- a/modules/gui/macosx/open.m
+++ b/modules/gui/macosx/open.m
@@ -38,6 +38,9 @@
 #include <IOKit/storage/IOMedia.h>
 #include <IOKit/storage/IOCDMedia.h>
 #include <IOKit/storage/IODVDMedia.h>
+#import <Cocoa/Cocoa.h>
+#import <QTKit/QTKit.h>
+#import <Foundation/NSobject.h>
 
 #import "intf.h"
 #import "playlist.h"
@@ -47,14 +50,14 @@
 
 #include <vlc_url.h>
 
+NSArray               *qtkvideoDevices;
 #define setEyeTVUnconnected \
-[o_capture_lbl setStringValue: _NS("No device connected")]; \
-[o_capture_long_lbl setStringValue: _NS("VLC could not detect any EyeTV compatible device.\n\nCheck the device's connection, make sure that the latest EyeTV software is installed and try again.")]; \
+[o_capture_lbl setStringValue: _NS("No device is selected")]; \
+[o_capture_long_lbl setStringValue: _NS("Any device is not selected.\n\nChose abailable device in above pull-down menu\n.")]; \
 [o_capture_lbl displayIfNeeded]; \
 [o_capture_long_lbl displayIfNeeded]; \
 [self showCaptureView: o_capture_label_view]
 
-
 /*****************************************************************************
  * GetEjectableMediaOfClass
  *****************************************************************************/
@@ -219,6 +222,7 @@ static VLCOpen *_o_sharedMainInstance = nil;
     [o_eyetv_chn_bgbar setUsesThreadedAnimation: YES];
 
     [o_capture_mode_pop removeAllItems];
+    [o_capture_mode_pop addItemWithTitle: @"None"];
     [o_capture_mode_pop addItemWithTitle: @"iSight"];
     [o_capture_mode_pop addItemWithTitle: _NS("Screen")];
     [o_capture_mode_pop addItemWithTitle: @"EyeTV"];
@@ -239,6 +243,27 @@ static VLCOpen *_o_sharedMainInstance = nil;
     [o_eyetv_launchEyeTV_btn setTitle: _NS("Launch EyeTV now")];
     [o_eyetv_getPlugin_btn setTitle: _NS("Download Plugin")];
 
+    [self qtkvideoDevices];
+    [o_qtk_device_pop removeAllItems];
+    msg_Dbg( VLCIntf, "Found %d capture devices", [qtkvideoDevices count] );
+    if([qtkvideoDevices count] == 0){
+        [o_qtk_device_pop addItemWithTitle: @"None"];
+        [qtk_currdevice_uid release];
+    }else {
+        if (!qtk_currdevice_uid) {
+            qtk_currdevice_uid = [[[QTCaptureDevice defaultInputDeviceWithMediaType: QTMediaTypeVideo] uniqueID]
+                                                                stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
+        }
+        for(int ivideo = 0; ivideo < [qtkvideoDevices count]; ivideo++){
+            QTCaptureDevice *qtk_device;
+            qtk_device = [qtkvideoDevices objectAtIndex:ivideo];
+            [o_qtk_device_pop addItemWithTitle: [qtk_device localizedDisplayName]];
+            if([[[qtk_device uniqueID]stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] isEqualToString:qtk_currdevice_uid]){
+                [o_qtk_device_pop selectItemAtIndex:ivideo];
+            }
+        }
+    }
+
     [self setSubPanel];
 
     [[NSNotificationCenter defaultCenter] addObserver: self
@@ -444,6 +469,15 @@ static VLCOpen *_o_sharedMainInstance = nil;
     }
 }
 
+- (IBAction)qtkChanged:(id)sender
+{
+    msg_Dbg( VLCIntf, "Changed UID: old %s", [qtk_currdevice_uid UTF8String] );
+    qtk_currdevice_uid = [[[qtkvideoDevices objectAtIndex:[o_qtk_device_pop indexOfSelectedItem]] uniqueID]
+                          stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
+    msg_Dbg( VLCIntf, "Changed UID: new %s", [qtk_currdevice_uid UTF8String] );
+    [o_mrl setStringValue:[NSString stringWithFormat:@"qtcapture://%@", qtk_currdevice_uid]];
+}
+
 - (void)tabView:(NSTabView *)o_tv didSelectTabViewItem:(NSTabViewItem *)o_tvi
 {
     NSString *o_label = [o_tvi label];
@@ -549,7 +583,6 @@ static VLCOpen *_o_sharedMainInstance = nil;
 - (void)openCapture
 {
     [self openCaptureModeChanged: nil];
-    [self showCaptureView: o_capture_label_view];
     [self openTarget: 3];
 }
 
@@ -640,7 +673,7 @@ static VLCOpen *_o_sharedMainInstance = nil;
             psz_class = kIOCDMediaClass;
             o_disc = o_type;
             b_no_menus = NO; b_title_chapter = YES;
-		}
+        }
         else if ( [o_type isEqualToString: _NS("Audio CD")])
         {
             psz_class = kIOCDMediaClass;
@@ -661,7 +694,7 @@ static VLCOpen *_o_sharedMainInstance = nil;
  
             if ( i_devices )
             {
-				for( int i = 0; i < i_devices; i++ )
+                for( int i = 0; i < i_devices; i++ )
                 {
                     [o_disc_device
                         addItemWithObjectValue: [o_devices objectAtIndex: i]];
@@ -746,7 +779,7 @@ static VLCOpen *_o_sharedMainInstance = nil;
             o_mrl_string = [NSString stringWithFormat: @"dvdread://%@@%i:%i-",
                             o_device, i_title, i_chapter];
         else
-			o_mrl_string = [NSString stringWithFormat: @"dvdnav://%@",
+            o_mrl_string = [NSString stringWithFormat: @"dvdnav://%@",
                             o_device];
             
     }
@@ -756,7 +789,7 @@ static VLCOpen *_o_sharedMainInstance = nil;
             o_mrl_string = [NSString stringWithFormat: @"dvdread://%@@%i:%i",
                             o_videots, i_title, i_chapter];
         else
-			o_mrl_string = [NSString stringWithFormat: @"dvdnav://%@",
+            o_mrl_string = [NSString stringWithFormat: @"dvdnav://%@",
                             o_videots];            
     }
 
@@ -1030,15 +1063,24 @@ static VLCOpen *_o_sharedMainInstance = nil;
         [o_screen_top_fld setIntValue: config_GetInt( p_intf, "screen-top" )];
         [o_screen_follow_mouse_ckb setIntValue: config_GetInt( p_intf, "screen-follow-mouse" )];
     }
+    else if([[[o_capture_mode_pop selectedItem] title] isEqualToString: @"None"])
+    {
+        setEyeTVUnconnected;
+        [o_mrl setStringValue: @""];
+    }
     else if( [[[o_capture_mode_pop selectedItem] title] isEqualToString: @"iSight"] )
     {
-        [o_capture_lbl setStringValue: _NS("iSight Capture Input")];
-        [o_capture_long_lbl setStringValue: _NS("This facility allows you to process your iSight's input signal.\n\nNo settings are available in this version, so you will be provided a 640px*480px raw video stream.\n\nLive Audio input is not supported.")];
-        [o_capture_lbl displayIfNeeded];
-        [o_capture_long_lbl displayIfNeeded];
-        
-        [self showCaptureView: o_capture_label_view];
-        [o_mrl setStringValue: @"qtcapture://"];
+        [self showCaptureView: o_qtk_view];
+        [o_qtk_lbl setStringValue: _NS("iSight Capture Input")];
+        [o_qtk_long_lbl setStringValue: _NS("This facility allows you to process your iSight's input signal.\n\nNo settings are available in this version, so you will be provided a 640px*480px raw video stream.\n\nLive Audio input is not supported.")];
+        [o_qtk_lbl displayIfNeeded];
+        [o_qtk_long_lbl displayIfNeeded];
+
+        if(!qtk_currdevice_uid){
+            [o_mrl setStringValue: @""];
+        }else{
+            [o_mrl setStringValue:[NSString stringWithFormat:@"qtcapture://%@", qtk_currdevice_uid]];
+        }
     }
 }
 
@@ -1241,6 +1283,19 @@ static VLCOpen *_o_sharedMainInstance = nil;
     }
 }
 
+- (NSArray *)qtkvideoDevices
+{
+    if (!qtkvideoDevices)
+        [self qtkrefreshDevices];
+    return qtkvideoDevices;
+}
+
+- (void)qtkrefreshDevices
+{
+    [qtkvideoDevices release];
+    qtkvideoDevices = [[[QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeVideo] arrayByAddingObjectsFromArray:[QTCaptureDevice inputDevicesWithMediaType:QTMediaTypeMuxed]] retain];
+}
+
 @end
 
 @implementation VLCOpenTextField
